from slxauth.utils import login_using_token


class TokenAuthMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.

        if not request.user.is_authenticated():
            access_token = request.COOKIES.get('access_token')
            expiration_time = request.COOKIES.get('expiration_time')
            refresh_token = request.COOKIES.get('refresh_token')

            if access_token and expiration_time and refresh_token:
                login_using_token(request, access_token, expiration_time, refresh_token)

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response




# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-10-09 15:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0002_auto_20171009_1719'),
    ]

    operations = [
        migrations.AddField(
            model_name='myuser',
            name='crm_contact_id',
            field=models.UUIDField(blank=True, null=True, verbose_name='CRM contact id'),
        ),
    ]



from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
from django.contrib.auth.models import PermissionsMixin
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _


class UserManager(BaseUserManager):
    use_in_migrations = True

    def _create_user(self, email, password, **extra_fields):
        """
        Creates and saves a User with the given username, email and password.
        """
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email=None, password=None, **extra_fields):
        extra_fields.setdefault('is_staff', False)
        extra_fields.setdefault('is_superuser', False)
        return self._create_user(email, password, **extra_fields)

    def create_superuser(self, email, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')

        return self._create_user(email, password, **extra_fields)


class AbstractSolarluxUser(AbstractBaseUser, PermissionsMixin):
    """
    An abstract base class implementing a fully featured User model with
    admin-compliant permissions.
    Email and password are required. Other fields are optional.
    """
    email = models.EmailField(
        _('email address'),
        unique=True,
        help_text=_('Required.'),
        error_messages={
            'unique': _("A user with that email already exists."),
        },
    )

    title = models.CharField(_('title'), max_length=20, blank=True)
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=150, blank=True)

    customer_no = models.IntegerField(_('customer no'), null=True, blank=True)
    customer_name = models.CharField(_('customer name'), max_length=200, blank=True)

    CUSTOMER_CATEGORY_CHOICES = (
        ('ACTIVE_PARTNER', _('active partner')),
        ('BI_FOLDING_DOOR_SPECIALIST', _('bi-folding-door specialist')),
        ('DEALER', _('dealer')),
        ('ARCHITECT_PLANNER', _('architect/planner')),
        ('HOUSING_SOCIETY', _('housing society')),
        ('PRIVATE_CUSTOMER', _('private customer')),
        ('OTHERS', _('others')),
    )

    customer_category = models.CharField(_('customer category'), choices=CUSTOMER_CATEGORY_CHOICES, max_length=200, blank=True)

    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this admin site.'),
    )
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )

    is_customer_admin = models.BooleanField(
        _('is customer admin'),
        default=False,
    )

    language = models.CharField(
        _('language'),
        max_length=2,
        null=True, blank=True)

    crm_contact_id = models.UUIDField(
        _('CRM contact id'),
        null=True, blank=True
    )

    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    um_id = models.IntegerField(_('user management id'), null=True, blank=True)
    avatar_attachment_id = models.IntegerField(_('avatar attachment id'), null=True, blank=True)
    company_logo_attachment_id = models.IntegerField(_('company logo attachment id'), null=True, blank=True)

    objects = UserManager()

    EMAIL_FIELD = 'email'
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    def clean(self):
        super().clean()
        self.email = self.__class__.objects.normalize_email(self.email)

    def get_full_name(self):
        """
        Return the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        """Return the short name for the user."""
        return self.first_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Send an email to this user."""
        send_mail(subject, message, from_email, [self.email], **kwargs)





import logging

import requests
from django.conf import settings
from django.contrib.auth.views import LogoutView
from django.http import HttpResponseRedirect, HttpResponseForbidden
from django.urls import reverse
from django.views import View
from requests.auth import HTTPBasicAuth

from slxauth.utils import login_using_token

logger = logging.getLogger(__name__)


def get_callback_uri(request):
    return request.build_absolute_uri(reverse('slxauth:callback'))

class LoginView(View):
    """ This is the view that django opens """
    def get(self, request):
        authorize_url = '%s/authorize?client_id=%s&client_secret=%s&response_type=code&redirect_uri=%s/' % (
            settings.OAUTH_URL,
            settings.OAUTH_CLIENT_ID,
            settings.OAUTH_CLIENT_SECRET,
            get_callback_uri(request)
        )
        logger.debug('redirecting to %s' % authorize_url)
        return HttpResponseRedirect(redirect_to=authorize_url)


class CallbackView(View):
    """ This handles the callback from the OAuth2 Auth server.
    Users will be redirected to this view after successful login.
    """
    def get(self, request):
        code = request.GET['code']
        logger.debug('received callback code %s' % code)
        url = '%s/token?grant_type=authorization_code&code=%s&redirect_uri=%s/' % (settings.OAUTH_URL, code, get_callback_uri(request))
        
        logger.debug('validating code using %s' % url)

        authresponse = requests.post(
            url,
            auth=HTTPBasicAuth(settings.OAUTH_CLIENT_ID, settings.OAUTH_CLIENT_SECRET),
            headers={'Content-Type': 'application/x-www-form-urlencoded'})

        logger.debug('response: %s' % authresponse)
        logger.debug('response content: %s' % authresponse.content)

        if authresponse.status_code == 200:
            data = authresponse.json()

            if 'access_token' in data and 'expires_in' in data and 'refresh_token' in data:
                access_token = data['access_token']
                expiration_time = data['expires_in']
                refresh_token = data['refresh_token']

                response = HttpResponseRedirect('%s/' % settings.SCRIPT_NAME) # todo: redirect to initially requested page
                response.set_cookie('access_token', access_token)
                response.set_cookie('expiration_time', expiration_time)
                response.set_cookie('refresh_token', refresh_token)

                login_using_token(request, access_token, expiration_time, refresh_token)
                return response

        return HttpResponseForbidden()


class LogoutView(LogoutView):
    def dispatch(self, request, *args, **kwargs):
        response = super(LogoutView, self).dispatch(request, *args, **kwargs)

        # clear the SSO cookies to logout from all portal apps
        response.set_cookie('access_token', max_age=0, domain=settings.OAUTH_COOKIE_DOMAIN)
        response.set_cookie('expiration_time', max_age=0, domain=settings.OAUTH_COOKIE_DOMAIN)
        response.set_cookie('refresh_token', max_age=0, domain=settings.OAUTH_COOKIE_DOMAIN)
        return response




import logging

import jwt
import requests
from django.conf import settings
from django.contrib.auth import get_user_model, login
from django.contrib.auth.models import Group


logger = logging.getLogger(__name__)


def get_unsaved_user_from_token(access_token):
    extra_data = jwt.decode(access_token, verify=False)

    user_model = get_user_model()
    user, created = user_model.objects.get_or_create(email=extra_data['email'])
    user.customer_no = extra_data['customerNumber']
    user.customer_name = extra_data['customerName']
    user.last_name = extra_data['lastName']
    user.first_name = extra_data['firstName']
    user.title = extra_data['title']
    user.is_customer_admin = extra_data['isCustomerAdmin']
    user.language = extra_data['language']
    user.crm_contact_id = extra_data['crmContactId']
    user.um_id = extra_data['id']

    for role in extra_data['authorities']:
        group, created = Group.objects.get_or_create(name=role)
        user.groups.add(group)

    return user

def get_unsaved_user_from_token_and_portal_config(access_token, portal_config):
    user = get_unsaved_user_from_token(access_token)

    if 'fetch-user-info-url' in portal_config:
        user_profile_response = requests.get(portal_config['fetch-user-info-url'], headers={'Authorization': 'Bearer %s' % access_token})
        if user_profile_response.status_code == 200:
            user_profile = user_profile_response.json()
            user.avatar_attachment_id = user_profile['avatarAttachmentId']
            user.company_logo_attachment_id = user_profile['companyLogoAttachmentId']

    return user

def login_using_token(request, access_token, expiration_time, refresh_token):

    try:
        # load more infos...
        portal_config_response = requests.get(settings.PORTAL_CONFIG_URL, cookies={
            'access_token': access_token,
            'expiration_time': str(expiration_time),
            'refresh_token': refresh_token,
        })
    except Exception as e:
        logger.error('could not load remote portal config: %s' % e)

    if portal_config_response and portal_config_response.status_code == 200:
        portal_config = portal_config_response.json()
        user = get_unsaved_user_from_token_and_portal_config(access_token, portal_config)
    else:
        user = get_unsaved_user_from_token(access_token)
    user.save()
    login(request, user)

